<?php
// Створення тимчасової папки
$tempDir = 'temp';
if (!is_dir($tempDir)) {
    mkdir($tempDir, 0777, true);
}

// Видалення файлів старших за 1 годину
foreach (glob("$tempDir/*") as $file) {
    if (filemtime($file) < time() - 3600) { // 3600 секунд = 1 година
        unlink($file);
    }
}

// Кодування/Декодування
$encodeResult = '';
if ($_SERVER['REQUEST_METHOD'] === 'POST' && isset($_POST['encode_text'], $_POST['encode_method'])) {
    $text = $_POST['encode_text'];
    $method = $_POST['encode_method'];
    switch ($method) {
        case 'base64_encode': $encodeResult = base64_encode($text); break;
        case 'base64_decode': $encodeResult = base64_decode($text); break;
        case 'ascii_encode': $encodeResult = bin2hex($text); break;
        case 'ascii_decode': $encodeResult = hex2bin($text); break;
        case 'url_encode': $encodeResult = urlencode($text); break;
        case 'url_decode': $encodeResult = urldecode($text); break;
        case 'md5_encode': $encodeResult = md5($text); break;
    }
}

// Шифрування
$encryptResult = '';
if ($_SERVER['REQUEST_METHOD'] === 'POST' && isset($_POST['encrypt_method'])) {
    $text = $_POST['encrypt_text'] ?? '';
    $key = $_POST['key'] ?? '';
    $method = $_POST['encrypt_method'];

    // Обробка завантаження зображення для стеганографії
    $imagePath = null;
    if (in_array($method, ['stegano_encrypt', 'stegano_decrypt']) && isset($_FILES['image']) && $_FILES['image']['error'] === UPLOAD_ERR_OK) {
        $imagePath = "$tempDir/" . uniqid() . '.' . pathinfo($_FILES['image']['name'], PATHINFO_EXTENSION);
        move_uploaded_file($_FILES['image']['tmp_name'], $imagePath);
    }

    switch ($method) {
        case 'aes_encrypt':
            if (empty($text)) {
                $encryptResult = 'Помилка: введіть текст';
            } elseif (empty($key)) {
                $encryptResult = 'Помилка: введіть ключ';
            } else {
                $iv = openssl_random_pseudo_bytes(openssl_cipher_iv_length('aes-256-cbc'));
                $encryptResult = openssl_encrypt($text, 'aes-256-cbc', $key, 0, $iv);
                $encryptResult = base64_encode($iv . $encryptResult);
            }
            break;
        case 'aes_decrypt':
            if (empty($text)) {
                $encryptResult = 'Помилка: введіть текст';
            } elseif (empty($key)) {
                $encryptResult = 'Помилка: введіть ключ';
            } else {
                $data = base64_decode($text);
                $ivLength = openssl_cipher_iv_length('aes-256-cbc');
                $iv = substr($data, 0, $ivLength);
                $encrypted = substr($data, $ivLength);
                $encryptResult = openssl_decrypt($encrypted, 'aes-256-cbc', $key, 0, $iv);
            }
            break;
        case 'stegano_encrypt':
            if (empty($text)) {
                $encryptResult = 'Помилка: введіть текст';
            } elseif (!$imagePath) {
                $encryptResult = 'Помилка: завантажте зображення (PNG, JPEG, BMP, GIF, WebP)';
            } else {
                $imageType = exif_imagetype($imagePath);
                switch ($imageType) {
                    case IMAGETYPE_PNG: $image = imagecreatefrompng($imagePath); break;
                    case IMAGETYPE_JPEG: $image = imagecreatefromjpeg($imagePath); break;
                    case IMAGETYPE_BMP: $image = imagecreatefrombmp($imagePath); break;
                    case IMAGETYPE_GIF: $image = imagecreatefromgif($imagePath); break;
                    case IMAGETYPE_WEBP: $image = imagecreatefromwebp($imagePath); break;
                    default: $encryptResult = 'Помилка: підтримується лише PNG, JPEG, BMP, GIF або WebP'; break;
                }
                if (isset($image)) {
                    $width = imagesx($image);
                    $height = imagesy($image);
                    $binaryMessage = '';
                    foreach (str_split($text) as $char) {
                        $binaryMessage .= sprintf('%08b', ord($char));
                    }
                    $messageIndex = 0;
                    for ($y = 0; $y < $height && $messageIndex < strlen($binaryMessage); $y++) {
                        for ($x = 0; $x < $width && $messageIndex < strlen($binaryMessage); $x++) {
                            $rgb = imagecolorat($image, $x, $y);
                            $r = ($rgb >> 16) & 0xFF;
                            $g = ($rgb >> 8) & 0xFF;
                            $b = $rgb & 0xFF;
                            $b = ($b & 0xFE) | (int)$binaryMessage[$messageIndex];
                            $messageIndex++;
                            imagesetpixel($image, $x, $y, imagecolorallocate($image, $r, $g, $b));
                        }
                    }
                    ob_start();
                    imagepng($image);
                    $encryptResult = 'data:image/png;base64,' . base64_encode(ob_get_clean());
                    imagedestroy($image);
                }
            }
            break;
        case 'stegano_decrypt':
            if (!$imagePath) {
                $encryptResult = 'Помилка: завантажте зображення (PNG, JPEG, BMP, GIF, WebP)';
            } else {
                $imageType = exif_imagetype($imagePath);
                switch ($imageType) {
                    case IMAGETYPE_PNG: $image = imagecreatefrompng($imagePath); break;
                    case IMAGETYPE_JPEG: $image = imagecreatefromjpeg($imagePath); break;
                    case IMAGETYPE_BMP: $image = imagecreatefrombmp($imagePath); break;
                    case IMAGETYPE_GIF: $image = imagecreatefromgif($imagePath); break;
                    case IMAGETYPE_WEBP: $image = imagecreatefromwebp($imagePath); break;
                    default: $encryptResult = 'Помилка: підтримується лише PNG, JPEG, BMP, GIF або WebP'; break;
                }
                if (isset($image)) {
                    $width = imagesx($image);
                    $height = imagesy($image);
                    $binaryMessage = '';
                    for ($y = 0; $y < $height; $y++) {
                        for ($x = 0; $x < $width; $x++) {
                            $rgb = imagecolorat($image, $x, $y);
                            $b = $rgb & 0xFF;
                            $binaryMessage .= $b & 1;
                        }
                    }
                    $encryptResult = '';
                    for ($i = 0; $i < strlen($binaryMessage); $i += 8) {
                        $byte = substr($binaryMessage, $i, 8);
                        if (strlen($byte) === 8) {
                            $encryptResult .= chr(bindec($byte));
                        }
                    }
                    imagedestroy($image);
                }
            }
            break;
    }
}
?>

<!DOCTYPE html>
<html lang="uk">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <meta name="description" content="Сервіс кодування, декодування та шифрування">
    <title>Кодування та Шифрування</title>
    <style>
        body {
            background: #e0e0e0;
            color: #333;
            font-family: Arial, sans-serif;
            margin: 0;
            padding: 0;
        }
        header, main {
            padding: 20px;
            background: #f5f5f5;
        }
        h1, h2 {
            color: #555;
        }
        form {
            margin: 10px 0;
        }
        input, textarea, select, button {
            display: block;
            width: 100%;
            margin: 5px 0;
            padding: 8px;
            border: 1px solid #888;
            border-radius: 3px;
            box-sizing: border-box;
        }
        button {
            background: #888;
            color: #fff;
            cursor: pointer;
        }
        button:hover {
            background: #666;
        }
        img {
            max-width: 100%;
            margin-top: 10px;
        }
        @media (max-width: 600px) {
            header, main {
                padding: 10px;
            }
            h1 {
                font-size: 1.5em;
            }
            input, textarea, select, button {
                font-size: 14px;
            }
        }
    </style>
    <script>
        function updateForm() {
            const method = document.querySelector('select[name="encrypt_method"]').value;
            const isStegano = method.includes('stegano');
            const isDecrypt = method === 'stegano_decrypt';
            document.querySelector('input[name="key"]').disabled = isStegano;
            document.querySelector('input[name="image"]').disabled = !isStegano;
            document.querySelector('input[name="image"]').required = isStegano;
            document.querySelector('textarea[name="encrypt_text"]').required = !isDecrypt;
        }
    </script>
</head>
<body onload="updateForm()">
    <header>
        <h1>Сервіс кодування та шифрування</h1>
    </header>
    <main>
        <h2>Кодування/Декодування</h2>
        <form method="POST">
            <textarea name="encode_text" placeholder="Введіть текст" required></textarea>
            <select name="encode_method">
                <option value="base64_encode">Base64 Encode</option>
                <option value="base64_decode">Base64 Decode</option>
                <option value="ascii_encode">ASCII Encode (Hex)</option>
                <option value="ascii_decode">ASCII Decode (Hex)</option>
                <option value="url_encode">URL Encode</option>
                <option value="url_decode">URL Decode</option>
                <option value="md5_encode">MD5 Encode</option>
            </select>
            <button type="submit">Виконати</button>
        </form>
        <?php if ($encodeResult): ?>
            <p>Результат: <strong><?php echo htmlspecialchars($encodeResult); ?></strong></p>
        <?php endif; ?>

        <h2>Шифрування/Розшифрування</h2>
        <form method="POST" enctype="multipart/form-data" onsubmit="updateForm()">
            <textarea name="encrypt_text" placeholder="Введіть текст"></textarea>
            <input type="text" name="key" placeholder="Ключ (для AES)">
            <select name="encrypt_method" onchange="updateForm()">
                <option value="aes_encrypt">AES Encrypt</option>
                <option value="aes_decrypt">AES Decrypt</option>
                <option value="stegano_encrypt">Steganography Encrypt (PNG/JPEG/BMP/GIF/WebP)</option>
                <option value="stegano_decrypt">Steganography Decrypt (PNG/JPEG/BMP/GIF/WebP)</option>
            </select>
            <input type="file" name="image" accept="image/png,image/jpeg,image/bmp,image/gif,image/webp">
            <button type="submit">Виконати</button>
        </form>
        <?php if ($encryptResult): ?>
            <?php if (strpos($encryptResult, 'data:image/png;base64,') === 0): ?>
                <p>Результат: <img src="<?php echo $encryptResult; ?>" alt="Steganography Result"></p>
            <?php else: ?>
                <p>Результат: <strong><?php echo htmlspecialchars($encryptResult); ?></strong></p>
            <?php endif; ?>
        <?php endif; ?>
    </main>
</body>
</html>